$! Program:	Operating-system dependent routines -- VMS version
$!		Builds the C-CLIENT library and servers
$!
$! Author:	Yehavi Bourvine, The Hebrew University of Jerusalem.
$!		Internet: Yehavi@VMS.huji.ac.il
$!
$! Modified:	Andy Harper, Kings College London.
$!		Internet: A.Harper@kcl.ac.uk
$!
$! Date:	2 August 1994
$! Last Edited:	24-JAN-1997
$!
$! Copyright 1994 by the University of Washington
$!
$!  Permission to use, copy, modify, and distribute this software and its
$! documentation for any purpose and without fee is hereby granted, provided
$! that the above copyright notice appears in all copies and that both the
$! above copyright notice and this permission notice appear in supporting
$! documentation, and that the name of the University of Washington not be
$! used in advertising or publicity pertaining to distribution of the software
$! without specific, written prior permission.	This software is made available
$! "as is", and
$! THE UNIVERSITY OF WASHINGTON DISCLAIMS ALL WARRANTIES, EXPRESS OR IMPLIED,
$! WITH REGARD TO THIS SOFTWARE, INCLUDING WITHOUT LIMITATION ALL IMPLIED
$! WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE, AND IN
$! NO EVENT SHALL THE UNIVERSITY OF WASHINGTON BE LIABLE FOR ANY SPECIAL,
$! INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
$! LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, TORT
$! (INCLUDING NEGLIGENCE) OR STRICT LIABILITY, ARISING OUT OF OR IN CONNECTION
$! WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
$!
$! Parameters:
$!	P1	Build options
$!			MULTINET or NETLIB	- Select TCP stack support
$!			LINK-ONLY		- Bypass compile stage
$!			VAXC			- Force use of VAXC
$!			GNUC			- Force use of GNUC
$!			DEBUG			- Include DEBUGGER
$!			OPTIMIZE		- Specify optimize only if no DEBUG
$!
$!		The following are specific to this module!!
$!			LOGTRACE		- Add support for writing debug
$!						  stuff via mm_dlog at a level
$!						  determined by IMAPD_DEBUG logical
$!			NETTRACE		- Logs network I/O to log file
$!						  at level determined via IMAPD_LOGLEVEL
$!
$!	P2	Modules to recompile
$!			module-list
$!
$ SocketLib = "SOCKET,CONNECT,BIND,LISTEN,SOCKET_READ,SOCKET_WRITE," + -
              "SOCKET_CLOSE,SELECT,ACCEPT,BCMP,BCOPY,BZERO," + -
              "GETHOSTBYNAME,GETHOSTBYADDR,GETPEERNAME,GETDTABLESIZE," + -
              "HTONS,HTONL,NTOHS,NTOHL,SEND,SENDTO,RECV,RECVFROM"
$!
$! Standard (+optional) defines
$ Defines   = "STDPROTO=""""vmsmailproto"""",VMSIO"
$ if f$locate("LOGTRACE",P1)         .lt. f$length(P1) then $ Defines=Defines+",LOGTRACE"
$ if f$locate("NETTRACE",P1)         .lt. f$length(P1) then $ Defines=Defines+",NETTRACE"
$
$ Includes = ""
$
$ CREATE LINKAGE.H
$ DECK
extern DRIVER imapdriver, nntpdriver, vmsmaildriver;
$ EOD
$ CREATE LINKAGE.C
$ DECK
    mail_link((DRIVER *)&imapdriver);
    mail_link((DRIVER *)&nntpdriver);
    mail_link((DRIVER *)&vmsmaildriver);
$ EOD
$ PURGE/nolog LINKAGE.*
$
$!
$ COPY/nolog OS_VMS.H OSDEP.H;	! Get the VMS specific header file in place
$ purge/nolog osdep.h
$
$! Set things up for the appropriate C compiler
$ CC_PREF = ""
$ LINK_OPT = ""
$
$! Find out which compilers we have
$ HAVEDECC = F$SEARCH("SYS$SYSTEM:DECC$COMPILER.EXE") .NES. ""
$ HAVEVAXC = F$SEARCH("SYS$SYSTEM:VAXC.EXE")          .NES. ""
$ HAVEGNUC = F$SEARCH("GNU_CC:[000000]GCC.EXE")       .NES. ""
$
$! ... and which to use in preference (if any)
$ FORCEVAXC = f$locate("VAXC",P1) .lt. f$length(P1)
$ FORCEGNUC = f$locate("GNUC",P1) .lt. f$length(P1)
$ if .not. (HAVEDECC .or. HAVEVAXC .or. HAVEGNUC)
$   then
$     write sys$output "%VMSBUILD-E-NOCC, No supported C compiler detected"
$     exit
$ endif
$
$! Auto-select, in order of preference, the best compiler available
$ COMPILER = ""
$ if (HAVEGNUC) then $ COMPILER = "GNU C"
$ if (HAVEVAXC) then $ COMPILER = "VAX C"
$ if (HAVEDECC) then $ COMPILER = "DEC C"
$
$! Multiple compilers specified?
$ if (FORCEVAXC .and. FORCEGNUC)
$   then
$     write sys$output "%VMSBUILD-E-AMBIG, Multiple C compilers specified"
$     exit
$ endif
$
$! Has user requested GNU C?
$ if FORCEGNUC
$   then
$     if .not. HAVEGNUC
$       then
$         write sys$output "%VMSBUILD-E-NOGNUC, GNUC selected but not installed"
$         exit
$     endif
$     COMPILER = "GNU C"
$ endif
$
$! Has user requested VAX C
$ if (FORCEVAXC)
$   then
$     if .not. HAVEVAXC
$       then
$         write sys$output "%VMSBUILD-E-NOVAXC, VAXC selected but not installed"
$         exit
$     endif
$     COMPILER = "VAX C"
$ endif
$
$ IF (COMPILER .eqs. "GNU C")
$   THEN
$       if f$type(GCC) .nes. ""
$         then
$           CC = GCC		! Defined foreign
$         else
$           CC = "GCC"		! Defined via SET COMMAND
$       endif
$
$	sysdef = "GNU_CC_INCLUDE:[SYS],SYS$LIBRARY"
$       netdef = "GNU_CC_INCLUDE:[VMS],SYS$LIBRARY"
$       arpdef = "GNU_CC_INCLUDE:[VMS],SYS$LIBRARY"
$
$	CC_PREF = CC_PREF + "/DEFINE=(''Defines')"
$       LINK_OPT = LINK_OPT + ",[---]VMS_GCC_LINK/OPTION"
$ ENDIF
$
$ IF (COMPILER .eqs. "VAX C")
$   THEN
$       if (HAVEDECC) then $ CC_PREF = CC_PREF + "/VAXC"
$       
$	sysdef = "SYS$LIBRARY"
$	netdef = "SYS$LIBRARY"
$	arpdef = "SYS$LIBRARY"
$
$	CC_PREF = CC_PREF + "/DEFINE=(''Defines')"
$!	CC_PREF = CC_PREF + "/LIST/SHOW=ALL"
$	LINK_OPT = LINK_OPT + ",[---]VMS_LINK/OPTION"
$ ENDIF
$
$ IF (COMPILER .eqs. "DEC C")
$   THEN
$       if (HAVEVAXC) then $ CC_PREF = CC_PREF + "/DECC"
$
$	sysdef = ""
$	netdef = ""
$	arpdef = ""
$
$	CC_PREF = CC_PREF + "/STANDARD=VAXC"
$	CC_PREF = CC_PREF + "/DEFINE=(''Defines')"
$!	CC_PREF = CC_PREF + "/LIST"
$ ENDIF
$
$! Initially have no defines for include files
$ IF F$TRNLNM("SYS")     .NES. "" THEN $ DEASSIGN SYS
$ IF F$TRNLNM("NETINET") .NES. "" THEN $ DEASSIGN NETINET
$ IF F$TRNLNM("ARPA")    .NES. "" THEN $ DEASSIGN ARPA
$
$!
$! DETERMINE NETWORK SUPPORT
$!
$ HAVENETLIB = f$trnlnm("NETLIB_DIR","LNM$SYSTEM") .nes. ""
$ HAVEMULTINET = f$trnlnm("MULTINET","LNM$SYSTEM") .nes. ""
$ FORCENETLIB = f$locate("NETLIB",P1) .lt. f$length(P1)
$ FORCEMULTINET = f$locate("MULTINET",P1) .lt. f$length(P1)
$ if (FORCEMULTINET .and. FORCENETLIB)
$   then
$     write sys$output "%VMSBUILD-E-CONFLTCP, Conflicting TCP stacks specified"
$     exit
$ endif
$
$! Pick best stack available
$ NETSTACK = "NO"
$ if (HAVEMULTINET) then $ NETSTACK = "MULTINET"
$ if (HAVENETLIB)   then $ NETSTACK = "NETLIB"
$
$ if (FORCEMULTINET)
$   then
$     if .not. HAVEMULTINET
$       then
$         write sys$output "%VMSBUILD-E-NOTCP, MULTINET tcp not installed"
$         exit
$     endif
$     NETSTACK = "MULTINET"
$ endif
$
$ if (FORCENETLIB)
$   then
$     if .not. HAVENETLIB
$       then
$         write sys$output "%VMSBUILD-E-NOTCP, NETLIB tcp not installed"
$         exit
$     endif
$     NETSTACK = "NETLIB"
$ endif
$
$! Set up for the selected network stack
$ if (NETSTACK .eqs. "NO")
$   then
$	COPY/nolog TCP_VMSN.C TCP_VMS.C;	! Default - no TcpIp support.
$	purge/nolog tcp_vms.c
$
$	CC_PREF    = CC_PREF + "/INCLUDE=([])"
$ endif
$
$ IF (NETSTACK .eqs. "MULTINET")
$   THEN
$
$	COPY/nolog TCP_VMSM.C TCP_VMS.C;	! Multinet support.
$       purge/nolog tcp_vms.c
$
$	LINK_OPT   = ",[---]VMS_MULTINET_LINK/OPTION" + LINK_OPT
$ ENDIF
$
$ IF (NETSTACK .eqs. "NETLIB")
$   THEN
$	COPY/nolog TCP_VMSL.C TCP_VMS.C;	! Netlib support.
$       purge/nolog tcp_vms.c
$
$	LINK_OPT   = ",[---]VMS_NETLIB_LINK/OPTION" + LINK_OPT
$ ENDIF
$!
$
$! Set up the includes based on the compiler and network requirements
$ CC_PREF = CC_PREF + "/INCLUDE=([]''includes')"
$
$! Set up the defines based on the compiler requirements and the network requirements
$ if (sysdef .nes. "") then $ define /nolog sys     'sysdef'
$ if (netdef .nes. "") then $ define /nolog netinet 'netdef'
$ if (arpdef .nes. "") then $ define /nolog arpa    'arpdef'
$ 
$
$! Set up debug option
$ if f$locate("DEBUG", P1) .LT F$LENGTH(P1)
$   then
$     CC_PREF = CC_PREF + "/DEBUG/NOOPTIMIZE"
$     LINK = "LINK/DEBUG"
$   else
$     if f$locate("OPTIMIZE",P1) .LT. F$LENGTH(P1)
$       then
$         CC_PREF = CC_PREF + "/OPTIMIZE"	! /OPT is NOT the default !
$       else
$         CC_PREF = CC_PREF + "/NOOPTIMIZE"
$     endif
$     LINK= "LINK"
$ endif
$
$ if f$locate("LINK-ONLY", P1) .lt. f$length(p1) then $ goto LINK
$ write sys$output "%VMSBUILD-I-CCLIENT, Building C-client library with ", COMPILER, " and ", NETSTACK, " TCP support"
$ if P2 .nes. ""
$   then
$      message = "$ CC" + CC_PREF + " " + P2
$      write/symbol sys$output message
$      CC'CC_PREF' 'P2'
$   else
$ SET VERIFY
$ CC'CC_PREF' OS_VMS
$ CC'CC_PREF' vms_mail
$ CC'CC_PREF' MAIL
$ CC'CC_PREF' SMTP
$ CC'CC_PREF' RFC822
$ CC'CC_PREF' NEWSRC
$ CC'CC_PREF' NNTP
$ CC'CC_PREF' nntpcvms
$ CC'CC_PREF' MISC
$ CC'CC_PREF' IMAP2
$ CC'CC_PREF' SM_VMS
$ CC'CC_PREF' TCP_VMS
$!
$ CC'CC_PREF' MTEST
$ CC'CC_PREF' /OBJ=[]IMAPD.OBJ [-.IMAPD]IMAPD
$ CC'CC_PREF' /OBJ=[]IPOP3D.OBJ [-.IPOPD]IPOP3D
$ SET NOVERIFY
$ endif
$!
$ write sys$output "%VMSBUILD-I-LIBRARY, Creating C-Client library"
$ LIBRARY/OBJECT/CREATE/INSERT C-CLIENT OS_VMS,vms_mail,MAIL,SMTP,RFC822,-
	NEWSRC,NNTP,nntpcvms,MISC,IMAP2,SM_VMS,TCP_VMS
$ PURGE/nolog C-CLIENT.OLB
$!
$LINK:
$ write sys$output "%VMSBUILD-I-LINK, Linking stand-alone MTEST utility"
$ LINK MTEST,IMAP2,MAIL,MISC,NNTP,newsrc,nntpcvms,OS_VMS,RFC822,SMTP,-
  SM_VMS,VMS_MAIL,TCP_VMS,SYS$INPUT:/OPTION'LINK_OPT'
$
$ write sys$output "%VMSBUILD-I-LINK, Linking stand-alone IMAPD server"
$ LINK IMAPD,IMAP2,MAIL,MISC,NEWSRC,NNTP,nntpcvms,OS_VMS,RFC822,SMTP,-
   SM_VMS,VMS_MAIL,TCP_VMS'LINK_OPT'
$
$ write sys$output "%VMSBUILD-I-LINK, Linking stand-alone POP3 server"
$ LINK IPOP3D,IMAP2,MAIL,MISC,NEWSRC,NNTPCVMS,NNTP,OS_VMS,RFC822,SMTP,-
   SM_VMS,VMS_MAIL,TCP_VMS'LINK_OPT'
$
$ PURGE/nolog *.OBJ
$ PURGE/nolog *.EXE
